/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package deezer

import (
	"errors"
	"strconv"

	"github.com/go-resty/resty/v2"
	"go.uploadedlobster.com/scotty/internal/version"
	"golang.org/x/oauth2"
)

const baseURL = "https://api.deezer.com/"
const MaxItemsPerGet = 1000
const DefaultRateLimitWaitSeconds = 5

type Client struct {
	HTTPClient *resty.Client
	token      oauth2.TokenSource
}

func NewClient(token oauth2.TokenSource) Client {
	client := resty.New()
	client.SetBaseURL(baseURL)
	client.SetHeader("Accept", "application/json")
	client.SetHeader("User-Agent", version.UserAgent())
	client.SetRetryCount(5)
	return Client{
		HTTPClient: client,
		token:      token,
	}
}

func (c Client) UserHistory(offset int, limit int) (result HistoryResult, err error) {
	const path = "/user/me/history"
	return listRequest[HistoryResult](c, path, offset, limit)
}

func (c Client) UserTracks(offset int, limit int) (TracksResult, error) {
	const path = "/user/me/tracks"
	return listRequest[TracksResult](c, path, offset, limit)
}

func (c Client) setToken(req *resty.Request) error {
	tok, err := c.token.Token()
	if err != nil {
		return err
	}

	req.SetQueryParam("access_token", tok.AccessToken)
	return nil
}

func listRequest[T Result](c Client, path string, offset int, limit int) (result T, err error) {
	request := c.HTTPClient.R().
		SetQueryParams(map[string]string{
			"index": strconv.Itoa(offset),
			"limit": strconv.Itoa(limit),
		}).
		SetResult(&result)
	err = c.setToken(request)
	if err != nil {
		return
	}
	response, err := request.Get(path)

	if !response.IsSuccess() {
		err = errors.New(response.String())
	} else if result.Error() != nil {
		err = errors.New(result.Error().Message)
	}
	return
}
